@@ -5,7 +5,9 @@ it's fairly self-explanatory.
 """
 
 import re
+import simplejson as json
 
+from dropbox.rest import ErrorResponse
 from dropbox.rest import RESTClient
 
 def format_path(path):
@@ -163,6 +165,7 @@ class DropboxClient(object):
             A dropbox.rest.ErrorResponse with an HTTP status of
                400: Bad request (may be due to many things; check e.error for details)
                404: No file was found at the given path, or the file that was there was deleted.
+               200: Request was okay but response was malformed in some way.
         """
         path = "/files/%s%s" % (self.session.root, format_path(from_path))
 
@@ -173,6 +176,46 @@ class DropboxClient(object):
         url, params, headers = self.request(path, params, method='GET', content_server=True)
         return RESTClient.request("GET", url, headers=headers, raw_response=True)
 
+    def get_file_and_metadata(self, from_path, rev=None):
+        """Download a file alongwith its metadata.
+
+        Acts as a thin wrapper around get_file() (see get_file() comments for
+        more details)
+
+        Args:
+            from_path: The path to the file to be downloaded.
+            rev: A previous rev value of the file to be downloaded. [optional]
+
+        Returns:
+            - An httplib.HTTPResponse that is the result of the request.
+            - A dictionary containing the metadata of the file (see
+              https://www.dropbox.com/developers/docs#metadata for details).
+
+        Raises:
+            A dropbox.rest.ErrorResponse with an HTTP status of
+               400: Bad request (may be due to many things; check e.error for details)
+               404: No file was found at the given path, or the file that was there was deleted.
+               200: Request was okay but response was malformed in some way.
+        """
+        file_res = self.get_file(from_path, rev)
+        metadata = DropboxClient.__parse_metadata_as_dict(file_res)
+
+        return file_res, metadata
+
+    @staticmethod
+    def __parse_metadata_as_dict(dropbox_raw_response):
+        """Parses file metadata from a raw dropbox HTTP response, raising a
+        dropbox.rest.ErrorResponse if parsing fails.
+        """
+        metadata = None
+        for header, header_val in dropbox_raw_response.getheaders():
+            if header.lower() == 'x-dropbox-metadata':
+                try:
+                    metadata = json.loads(header_val)
+                except ValueError:
+                    raise ErrorResponse(dropbox_raw_response)
+        if not metadata: raise ErrorResponse(dropbox_raw_response)
+        return metadata
 
     def file_copy(self, from_path, to_path):
         """Copy a file or folder to a new location.
@@ -374,7 +417,37 @@ class DropboxClient(object):
         url, params, headers = self.request(path, {'size': size}, method='GET', content_server=True)
         return RESTClient.request("GET", url, headers=headers, raw_response=True)
 
-    def search(self, path, query, file_limit=10000, include_deleted=False):
+    def thumbnail_and_metadata(self, from_path, size='large', format='JPEG'):
+        """Download a thumbnail for an image alongwith its metadata.
+
+        Acts as a thin wrapper around thumbnail() (see thumbnail() comments for
+        more details)
+
+        Args:
+            from_path: The path to the file to be thumbnailed.
+            size: A string describing the desired thumbnail size. See thumbnail()
+               for details.
+
+        Returns:
+            - An httplib.HTTPResponse that is the result of the request.
+            - A dictionary containing the metadata of the file whose thumbnail
+              was downloaded (see https://www.dropbox.com/developers/docs#metadata
+              for details).
+
+        Raises:
+            A dropbox.rest.ErrorResponse with an HTTP status of
+
+            - 400: Bad request (may be due to many things; check e.error for details)
+            - 404: No file was found at the given from_path, or files of that type cannot be thumbnailed.
+            - 415: Image is invalid and cannot be thumbnailed.
+            - 200: Request was okay but response was malformed in some way.
+        """
+        thumbnail_res = self.thumbnail(from_path, size, format)
+        metadata = DropboxClient.__parse_metadata_as_dict(thumbnail_res)
+
+        return thumbnail_res, metadata
+
+    def search(self, path, query, file_limit=1000, include_deleted=False):
         """Search directory for filenames matching query.
 
         Args:
@@ -383,7 +456,7 @@ class DropboxClient(object):
             query: The query to search on (minimum 3 characters).
 
             file_limit: The maximum number of file entries to return within a folder.
-               The server will return at max 10,000 files.
+               The server will return at max 1,000 files.
 
             include_deleted: Whether to include deleted files in search results.
 
@@ -437,7 +510,7 @@ class DropboxClient(object):
             'rev_limit': rev_limit,
             }
 
-        url, params, headers = self.request(path, params)
+        url, params, headers = self.request(path, params, method='GET')
 
         return RESTClient.GET(url, headers)
 
