@@ -1,17 +1,19 @@
 """
 A simple JSON REST request abstraction layer that is used by the
-dropbox.client and dropbox.session modules. You shouldn't need to use this
-directly unless you're implementing new methods we haven't added to the SDK yet.
+dropbox.client and dropbox.session modules. You shouldn't need to use this.
 """
 
 import httplib
+import os
+import pkg_resources
+import re
 import simplejson as json
 import socket
+import ssl
 import urllib
 import urlparse
-import os
 
-SDK_VERSION = "1.2"
+SDK_VERSION = "1.3"
 
 class RESTClient(object):
     """
@@ -59,7 +61,7 @@ class RESTClient(object):
             headers["Content-type"] = "application/x-www-form-urlencoded"
 
         host = urlparse.urlparse(url).hostname
-        conn = httplib.HTTPSConnection(host, 443)
+        conn = ProperHTTPSConnection(host, 443)
 
         try:
 
@@ -103,6 +105,8 @@ class RESTClient(object):
 
         except socket.error, e:
             raise RESTSocketError(host, e)
+        except CertificateError, e:
+            raise RESTSocketError(host, "SSL certificate error: " + e)
 
         r = conn.getresponse()
         if r.status != 200:
@@ -153,8 +157,10 @@ class RESTSocketError(socket.error):
 
 class ErrorResponse(Exception):
     """
-    Raised by dropbox.rest.RESTClient.request for requests that return a non-200
-    HTTP response or have a non-JSON response body.
+    Raised by dropbox.rest.RESTClient.request for requests that:
+    - Return a non-200 HTTP response, or
+    - Have a non-JSON response body, or
+    - Have a malformed/missing header in the response.
 
     Most errors that Dropbox returns will have a error field that is unpacked and
     placed on the ErrorResponse exception. In some situations, a user_error field
@@ -166,6 +172,7 @@ class ErrorResponse(Exception):
         self.status = http_resp.status
         self.reason = http_resp.reason
         self.body = http_resp.read()
+        self.headers = http_resp.getheaders()
 
         try:
             body = json.loads(self.body)
@@ -184,6 +191,102 @@ class ErrorResponse(Exception):
         elif not self.body:
             msg = self.reason
         else:
-            msg = "Error parsing response body: %s" % self.body
+            msg = "Error parsing response body or headers: " +\
+                  "Body - %s Headers - %s" % (self.body, self.headers)
 
         return "[%d] %s" % (self.status, repr(msg))
+
+TRUSTED_CERT_FILE = pkg_resources.resource_filename(__name__, 'trusted-certs.crt')
+
+class ProperHTTPSConnection(httplib.HTTPConnection):
+    """
+    httplib.HTTPSConnection is broken because it doesn't do server certificate
+    validation.  This class does certificate validation by ensuring:
+       1. The certificate sent down by the server has a signature chain to one of
+          the certs in our 'trusted-certs.crt' (this is mostly handled by the 'ssl'
+          module).
+       2. The hostname in the certificate matches the hostname we're connecting to.
+    """
+
+    def __init__(self, host, port):
+        httplib.HTTPConnection.__init__(self, host, port)
+        self.ca_certs = TRUSTED_CERT_FILE
+        self.cert_reqs = ssl.CERT_REQUIRED
+
+    def connect(self):
+        sock = create_connection((self.host, self.port))
+        self.sock = ssl.wrap_socket(sock, cert_reqs=self.cert_reqs, ca_certs=self.ca_certs)
+        cert = self.sock.getpeercert()
+        hostname = self.host.split(':', 0)[0]
+        match_hostname(cert, hostname)
+
+class CertificateError(ValueError):
+    pass
+
+def _dnsname_to_pat(dn):
+    pats = []
+    for frag in dn.split(r'.'):
+        if frag == '*':
+            # When '*' is a fragment by itself, it matches a non-empty dotless
+            # fragment.
+            pats.append('[^.]+')
+        else:
+            # Otherwise, '*' matches any dotless fragment.
+            frag = re.escape(frag)
+            pats.append(frag.replace(r'\*', '[^.]*'))
+    return re.compile(r'\A' + r'\.'.join(pats) + r'\Z', re.IGNORECASE)
+
+def match_hostname(cert, hostname):
+    """Verify that *cert* (in decoded format as returned by
+    SSLSocket.getpeercert()) matches the *hostname*.  RFC 2818 rules
+    are mostly followed, but IP addresses are not accepted for *hostname*.
+
+    CertificateError is raised on failure. On success, the function
+    returns nothing.
+    """
+    if not cert:
+        raise ValueError("empty or no certificate")
+    dnsnames = []
+    san = cert.get('subjectAltName', ())
+    for key, value in san:
+        if key == 'DNS':
+            if _dnsname_to_pat(value).match(hostname):
+                return
+            dnsnames.append(value)
+    if not san:
+        # The subject is only checked when subjectAltName is empty
+        for sub in cert.get('subject', ()):
+            for key, value in sub:
+                # XXX according to RFC 2818, the most specific Common Name
+                # must be used.
+                if key == 'commonName':
+                    if _dnsname_to_pat(value).match(hostname):
+                        return
+                    dnsnames.append(value)
+    if len(dnsnames) > 1:
+        raise CertificateError("hostname %r doesn't match either of %s" % (hostname, ', '.join(map(repr, dnsnames))))
+    elif len(dnsnames) == 1:
+        raise CertificateError("hostname %r doesn't match %r" % (hostname, dnsnames[0]))
+    else:
+        raise CertificateError("no appropriate commonName or subjectAltName fields were found")
+
+def create_connection(address):
+    host, port = address
+    err = None
+    for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
+        af, socktype, proto, canonname, sa = res
+        sock = None
+        try:
+            sock = socket.socket(af, socktype, proto)
+            sock.connect(sa)
+            return sock
+
+        except socket.error, _:
+            err = _
+            if sock is not None:
+                sock.close()
+
+    if err is not None:
+        raise err
+    else:
+        raise socket.error("getaddrinfo returns an empty list")
