@@ -62,6 +62,14 @@ class DropboxSession(object):
         """
         self.token = oauth.OAuthToken(access_token, access_token_secret)
 
+    def set_request_token(self, request_token, request_token_secret):
+        """Attach an request token to the DropboxSession.
+
+        Note that the reuest 'token' is made up of both a token string
+        and a secret string.
+        """
+        self.token = oauth.OAuthToken(request_token, request_token_secret)
+
     def build_path(self, target, params=None):
         """Build the path component for an API URL.
 
