@@ -79,7 +79,7 @@ class DropboxTerm(cmd.Cmd):
     @command()
     def do_cat(self, path):
         """display the contents of a file"""
-        f = self.api_client.get_file(self.current_path + "/" + path)
+        f, metadata = self.api_client.get_file_and_metadata(self.current_path + "/" + path)
         self.stdout.write(f.read())
         self.stdout.write("\n")
 
@@ -105,11 +105,41 @@ class DropboxTerm(cmd.Cmd):
         f = self.api_client.account_info()
         pprint.PrettyPrinter(indent=2).pprint(f)
 
-    @command()
+    @command(login_required=False)
     def do_exit(self):
         """exit"""
         return True
 
+    @command()
+    def do_get(self, from_path, to_path):
+        """
+        Copy file from Dropbox to local file and print out out the metadata.
+
+        Examples:
+        Dropbox> get file.txt ~/dropbox-file.txt
+        """
+        to_file = open(os.path.expanduser(to_path), "wb")
+
+        f, metadata = self.api_client.get_file_and_metadata(self.current_path + "/" + from_path)
+        print 'Metadata:', metadata
+        to_file.write(f.read())
+
+    @command()
+    def do_thumbnail(self, from_path, to_path, size='large', format='JPEG'):
+        """
+        Copy an image file's thumbnail to a local file and print out the
+        file's metadata.
+
+        Examples:
+        Dropbox> thumbnail file.txt ~/dropbox-file.txt medium PNG
+        """
+        to_file = open(os.path.expanduser(to_path), "wb")
+
+        f, metadata = self.api_client.thumbnail_and_metadata(
+                self.current_path + "/" + from_path, size, format)
+        print 'Metadata:', metadata
+        to_file.write(f.read())
+
     @command()
     def do_put(self, from_path, to_path):
         """
@@ -122,7 +152,7 @@ class DropboxTerm(cmd.Cmd):
 
         self.api_client.put_file(self.current_path + "/" + to_path, from_file)
 
-    @command()
+    @command(login_required=False)
     def do_help(self):
         # Find every "do_" attribute with a non-empty docstring and print
         # out the docstring.
